using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using UnityEngine;
using HarmonyLib;

public class NoPillars : Mod
{
    private readonly string HARMONY_ID = "TeigRolle.NoPillars";
    private Harmony harmonyInstance;

    // The Start() method is being called when your mod gets loaded.
    public void Start()
    {
        harmonyInstance = new Harmony(HARMONY_ID);
        harmonyInstance.PatchAll(Assembly.GetExecutingAssembly());
        RConsole.Log("NoPillars has been loaded!");
    }

    public static GameObject newFloorColliderPrefab;
    public static GameObject newFloorTriangularColliderPrefab;

    public void ChangeBlock()
    {
        string pathbcm = "blockcollisionmasks/blockcollisionmask_pillar";
        SO_BlockCollisionMask pillarMask = Traverse.Create(Resources.Load<ScriptableObject>(pathbcm))
            .GetValue<SO_BlockCollisionMask>();


        string pathbqt = "blockquadtype/quad_floor_empty";
        SO_BlockQuadType blockQuadType =
            Traverse.Create(Resources.Load<ScriptableObject>(pathbqt)).GetValue<SO_BlockQuadType>();

        List<Item_Base> list = Traverse.Create(typeof(ItemManager)).Field("allAvailableItems")
            .GetValue<List<Item_Base>>();

        foreach (Item_Base item_Base in list)
        {
            if (item_Base.UniqueIndex == 1 ||
                item_Base.UniqueIndex == 191 ||
                item_Base.UniqueIndex == 384 ||
                item_Base.UniqueIndex == 385)
            {
                Destroy(item_Base.settings_buildable.GetBlockPrefab(0).gameObject.GetComponent<StableComponent>());
                item_Base.settings_buildable.GetBlockPrefab(0).blockCollisionMask = pillarMask;
            }
        }

        Traverse.Create(typeof(ItemManager)).Field("allAvailableItems").SetValue(list);

        Item_Base floorItem = ItemManager.GetItemByIndex(1);
        Item_Base floorTriangularItem = ItemManager.GetItemByIndex(191);

        //Create for floor and apply to triangular later
        newFloorColliderPrefab = Instantiate(floorItem.settings_buildable.GetBlockPrefab(0).colliderPrefab);
        Destroy(newFloorColliderPrefab.GetComponent<StableComponent>());

        BlockSurface blockSurface = new BlockSurface();
        blockSurface.surfaceType = SurfaceType.All;
        blockSurface.dpsType = DPS.Ceiling;

        GameObject[] newFloorColliders = new GameObject[]
        {
            new GameObject("BuildquadCeilingNewR"),
            new GameObject("BuildquadCeilingNewL"),
            new GameObject("BuildquadCeilingNewB"),
            new GameObject("BuildquadCeilingNewF")
        };

        for (int i = 0; i < newFloorColliders.Length; i++)
        {
            newFloorColliders[i].layer = 8;
            newFloorColliders[i].transform.parent = newFloorColliderPrefab.transform;
            newFloorColliders[i].transform.localEulerAngles = new Vector3(0.0f, 90.0f, 0.0f);
            newFloorColliders[i].AddComponent<BoxCollider>().isTrigger = true;
            newFloorColliders[i].GetComponent<BoxCollider>().size = new Vector3(1.45f, 0.2f, 1.45f);
            newFloorColliders[i].AddComponent<BlockQuad>();
            newFloorColliders[i].GetComponent<BlockQuad>().quadType = blockQuadType;
            newFloorColliders[i].GetComponent<BlockQuad>().acceptableBuildSides = new BlockSurface[] { blockSurface };
            newFloorColliders[i].GetComponent<BlockQuad>().snapToQuadRotation = true;
        }

        newFloorColliders[0].transform.localPosition = new Vector3(1.5f, 0.0f, 0.0f);
        newFloorColliders[1].transform.localPosition = new Vector3(-1.5f, 0.0f, 0.0f);
        newFloorColliders[2].transform.localPosition = new Vector3(0.0f, 0.0f, 1.5f);
        newFloorColliders[3].transform.localPosition = new Vector3(0.0f, 0.0f, -1.5f);

        floorItem.settings_buildable.GetBlockPrefab(0).colliderPrefab = newFloorColliderPrefab;

        newFloorTriangularColliderPrefab = Instantiate(newFloorColliderPrefab);

        Destroy(newFloorTriangularColliderPrefab.transform.Find("BuildquadCeilingNewF").gameObject);
        Destroy(newFloorTriangularColliderPrefab.transform.Find("BuildquadCeilingNewR").gameObject);

        DontDestroyOnLoad(newFloorColliderPrefab);
        DontDestroyOnLoad(newFloorTriangularColliderPrefab);

        floorTriangularItem.settings_buildable.GetBlockPrefab(0).colliderPrefab = newFloorTriangularColliderPrefab;
    }

}

[HarmonyPatch(typeof(ColliderPrefabEnabler))]
[HarmonyPatch("AttachColliderPrefab")]
class ColliderPrefabEnablerPatch
{
    static void Prefix(ColliderPrefabEnabler __instance, Block block)
    {
        if (block == null || block.buildableItem == null) return;

        var uniqueIndex = block.buildableItem.UniqueIndex;

        if (uniqueIndex == 1 || uniqueIndex == 191 || uniqueIndex == 384 || uniqueIndex == 385)
        {
            if (NoPillars.newFloorColliderPrefab == null)
            {
                GameObject.FindObjectOfType<NoPillars>().ChangeBlock();
            }
            block.colliderPrefab = uniqueIndex == 191 || uniqueIndex == 385 ? NoPillars.newFloorTriangularColliderPrefab : NoPillars.newFloorColliderPrefab;
        }
    }
}


[HarmonyPatch(typeof(StableComponent), "IsStable")]
class StableComponentPatch
{
    static bool Prefix(StableComponent __instance, ref bool __result)
    {
        Block thisBlock = __instance.GetComponent<Block>();
        if (thisBlock == null)
        {
            thisBlock = __instance.transform.parent.GetComponent<Block>();
        }

        if (thisBlock != null)
        {
            if (thisBlock.buildableItem.UniqueIndex == 1 ||
                thisBlock.buildableItem.UniqueIndex == 191 ||
                thisBlock.buildableItem.UniqueIndex == 384 ||
                thisBlock.buildableItem.UniqueIndex == 385)
            {
                __result = true;
                return false;
            }
        }

        return true;
    }
}
